package com.jkcredit.illegal.info.service.impl;

import com.jkcredit.illegal.info.constant.CommonConstant;
import com.jkcredit.illegal.info.model.SendDingMsgResponseObject;
import com.jkcredit.illegal.info.service.DingRobotService;
import com.jkcredit.illegal.info.service.MonitorService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.time.LocalTime;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Service
@Slf4j
@RequiredArgsConstructor
public class MonitorServiceImpl implements MonitorService {
    private final StringRedisTemplate stringRedisTemplate;
    private final DingRobotService dingRobotService;
    private static final Double PROPORTION = 0.5;

    @Override
    public void requestMonitor() {
        LocalTime currentTime = LocalTime.now();
        LocalTime startTime = LocalTime.of(8, 0);
        LocalTime endTime = LocalTime.of(23, 0);
        boolean isWarnTime = !currentTime.isBefore(startTime) && !currentTime.isAfter(endTime);
        String serviceStatus = stringRedisTemplate.boundValueOps(CommonConstant.SERVICE_STATUS).get();

        // 业务可用状态，并且时间在8-23点之间，做业务异常判断
        if (CommonConstant.SERVICE_AVAILABLE.equals(serviceStatus) && isWarnTime) {
            warn();
        }
        // 业务不可用状态，做业务恢复判断
        if (CommonConstant.SERVICE_UN_AVAILABLE.equals(serviceStatus)) {
            recover();
        }

        stringRedisTemplate.delete(CommonConstant.TIME_OUT_COUNT);
        stringRedisTemplate.delete(CommonConstant.QUERY_FAILED_COUNT);
        stringRedisTemplate.delete(CommonConstant.QUERY_TOTAL_COUNT);
    }

    private void recover() {
        DecimalFormat decimalFormat = new DecimalFormat("0.00%");
        double timeOutCount = 0;
        double queryFailedCount = 0;
        double totalCount = 0;
        String timeOut = stringRedisTemplate.opsForValue().get(CommonConstant.TIME_OUT_COUNT);
        String queryFailed = stringRedisTemplate.opsForValue().get(CommonConstant.QUERY_FAILED_COUNT);
        String total = stringRedisTemplate.opsForValue().get(CommonConstant.QUERY_TOTAL_COUNT);
        if (timeOut != null) {
            timeOutCount = Double.parseDouble(timeOut);
        }
        if (queryFailed != null) {
            queryFailedCount = Double.parseDouble(queryFailed);
        }
        if (total != null) {
            totalCount = Double.parseDouble(total);
        }

        StringBuilder errorMessage = new StringBuilder();
        double timeOutProportion = timeOutCount / totalCount;
        double queryFailedProportion = queryFailedCount / totalCount;
        if (timeOutProportion < PROPORTION && queryFailedProportion < PROPORTION) {
            errorMessage.append("核验-数据源恢复, 超时比例").append(decimalFormat.format(timeOutProportion));
            errorMessage.append(", 异常比例").append(decimalFormat.format(queryFailedProportion));
            SendDingMsgResponseObject sendDingMsgResponseObject = dingRobotService.sendDingRobotMsg(errorMessage.toString());
            if (sendDingMsgResponseObject.getCode() != 1) {
                log.info("personIllegalInfoTask, 发送钉钉异常：核验-数据源恢复");
            } else {
                stringRedisTemplate.boundValueOps(CommonConstant.SERVICE_STATUS).set(CommonConstant.SERVICE_AVAILABLE);
            }
        }
    }

    private void warn() {
        DecimalFormat decimalFormat = new DecimalFormat("0.00%");
        double timeOutCount = 0;
        double queryFailedCount = 0;
        double totalCount = 0;
        String timeOut = stringRedisTemplate.opsForValue().get(CommonConstant.TIME_OUT_COUNT);
        String queryFailed = stringRedisTemplate.opsForValue().get(CommonConstant.QUERY_FAILED_COUNT);
        String total = stringRedisTemplate.opsForValue().get(CommonConstant.QUERY_TOTAL_COUNT);
        if (timeOut != null) {
            timeOutCount = Double.parseDouble(timeOut);
        }
        if (queryFailed != null) {
            queryFailedCount = Double.parseDouble(queryFailed);
        }
        if (total != null) {
            totalCount = Double.parseDouble(total);
        }

        StringBuilder errorMessage = new StringBuilder();
        double timeOutProportion = timeOutCount / totalCount;
        if (timeOutProportion >= PROPORTION) {
            errorMessage.append("核验-数据源超时, 超时比例 ").append(decimalFormat.format(timeOutProportion));
            SendDingMsgResponseObject sendDingMsgResponseObject = dingRobotService.sendDingRobotMsg(errorMessage.toString());
            if (sendDingMsgResponseObject.getCode() != 1) {
                log.info("personIllegalInfoTask, 发送钉钉异常：社会风险预警-人员核验接口失败-数据源超时");
            } else {
                stringRedisTemplate.boundValueOps(CommonConstant.SERVICE_STATUS).set(CommonConstant.SERVICE_UN_AVAILABLE);
            }
        }

        double queryFailedProportion = queryFailedCount / totalCount;
        if (queryFailedProportion >= 0.5) {
            errorMessage.append("核验-数据源返回结果异常，异常比例 ").append(decimalFormat.format(queryFailedProportion));
            SendDingMsgResponseObject sendDingMsgResponseObject = dingRobotService.sendDingRobotMsg(errorMessage.toString());
            if (sendDingMsgResponseObject.getCode() != 1) {
                log.info("personIllegalInfoTask, 发送钉钉异常：社会风险预警-人员核验接口失败-数据源返回结果异常");
            } else {
                stringRedisTemplate.boundValueOps(CommonConstant.SERVICE_STATUS).set(CommonConstant.SERVICE_UN_AVAILABLE);
            }
        }
    }

    @Override
    public void updateClientCertificate() {
        dingRobotService.sendDingRobotMsg("证书即将到期，请及时更换");
    }
}
